import {
  EnumType,
  ICrossFieldConditionsParams,
  ICrossFieldValidations,
  IFormFieldBase,
  IFormGroupBase,
  IJsonSchema,
  RegularBreakpoints,
} from './modelsCommon';

interface IValidationObjectV1<T extends EnumType> extends IJsonSchema<T> {
  crossFieldValidations: ICrossFieldValidations<T>;
}

export interface IFormFieldV1<T extends EnumType> extends IFormFieldBase<T> {
  grid?: RegularBreakpoints;
  renderCondition?: string;
  disabledCondition?: string;
}

export interface IFormGroupV1<T extends EnumType> extends IFormGroupBase<T> {
  children: Array<IFormFieldV1<T>>;
}

export interface IFormV1<T extends EnumType> {
  formatVersion: 1;
  validationObject: IValidationObjectV1<T>;
  children: Array<IFormFieldV1<T> | IFormGroupV1<T>>;
  submitButtonText: string;
  cancelButtonText?: string;
}

type ICrossFieldConditions<T extends EnumType> = {
  [key in T]?: ICrossFieldConditionsParams;
};

type IFormLayout<T extends EnumType> = {
  [key in T]?: RegularBreakpoints;
};

export interface IFormV2<T extends EnumType> {
  formatVersion: 2;
  validationObject: IJsonSchema<T>;
  crossFieldValidations: ICrossFieldValidations<T>;
  crossFieldConditions: ICrossFieldConditions<T>;
  formLayout: IFormLayout<T>;
  children: Array<IFormFieldBase<T> | IFormGroupBase<T>>;
  submitButtonText: string;
  cancelButtonText?: string;
}

export type IAllForms<T extends EnumType> = IFormV1<T> | IFormV2<T>;
